@@ -16,7 +16,6 @@ from portage.util import ensure_dirs
 
 class BlockerFileCollisionEmergeTestCase(TestCase):
     def testBlockerFileCollision(self):
-
         debug = False
 
         install_something = """
@@ -160,9 +159,8 @@ src_install() {
                 stdout = subprocess.PIPE
 
             for i, args in enumerate(test_commands):
-
                 if hasattr(args[0], "__call__"):
-                    self.assertTrue(args[0](), "callable at index {} failed".format(i))
+                    self.assertTrue(args[0](), f"callable at index {i} failed")
                     continue
 
                 if isinstance(args[0], dict):
@@ -185,7 +183,7 @@ src_install() {
                             sys.stderr.write(_unicode_decode(line))
 
                 self.assertEqual(
-                    os.EX_OK, proc.returncode, "emerge failed with args {}".format(args)
+                    os.EX_OK, proc.returncode, f"emerge failed with args {args}"
                 )
         finally:
             playground.debug = False
