@@ -58,7 +58,6 @@ class PopenPipeBlockingIOTestCase(TestCase):
         return consumer.getvalue().decode("ascii", "replace")
 
     def testPopenPipeBlockingIO(self):
-
         if threading is None:
             skip_reason = "threading disabled"
             self.portage_skip = "threading disabled"
@@ -69,5 +68,5 @@ class PopenPipeBlockingIOTestCase(TestCase):
             test_string = x * "a"
             output = self._testPipeReader(test_string)
             self.assertEqual(
-                test_string, output, "x = {}, len(output) = {}".format(x, len(output))
+                test_string, output, f"x = {x}, len(output) = {len(output)}"
             )
